@@ -14,14 +14,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""
-OpenADR Client for Python
-"""
-
 import asyncio
 import inspect
 import logging
 import ssl
+import sys
 import random
 from datetime import datetime, timedelta, timezone
 from functools import partial
@@ -151,9 +148,10 @@ class OpenADRClient:
             self.scheduler.shutdown()
         if self.report_queue_task:
             self.report_queue_task.cancel()
-        delayed_call_tasks = [task for task in asyncio.all_tasks() if task.get_name().startswith('DelayedCall')]
-        for task in delayed_call_tasks:
-            task.cancel()
+        if sys.version_info.minor > 8:
+            delayed_call_tasks = [task for task in asyncio.all_tasks() if task.get_name().startswith('DelayedCall')]
+            for task in delayed_call_tasks:
+                task.cancel()
         await self.client_session.close()
         await asyncio.sleep(0)
 
@@ -603,9 +601,12 @@ class OpenADRClient:
                 logger.info("The report is now complete with all the values. Will queue for sending.")
                 if self.allow_jitter:
                     delay = random.uniform(0, min(30, report_interval / 2))
+                    if sys.version_info.minor >= 8:
+                        name = {'name': f'DelayedCall-OutgoingReport-{utils.generate_id()}'}
+                    else:
+                        name = {}
                     self.loop.create_task(utils.delayed_call(func=self.pending_reports.put(outgoing_report),
-                                                             delay=delay),
-                                          name=f'DelayedCall-{utils.generate_id()}')
+                                                             delay=delay), **name)
                 else:
                     await self.pending_reports.put(self.incomplete_reports.pop(report_request_id))
             else:
@@ -615,9 +616,12 @@ class OpenADRClient:
             logger.info("Report will be sent now.")
             if self.allow_jitter:
                 delay = random.uniform(0, min(30, granularity.total_seconds() / 2))
+                if sys.version_info.minor >= 8:
+                    name = {'name': f'DelayedCall-OutgoingReport-{utils.generate_id()}'}
+                else:
+                    name = {}
                 self.loop.create_task(utils.delayed_call(func=self.pending_reports.put(outgoing_report),
-                                                         delay=delay),
-                                      name=f'DelayedCall-{utils.generate_id()}')
+                                                         delay=delay), **name)
             else:
                 await self.pending_reports.put(outgoing_report)
 
@@ -669,11 +673,6 @@ class OpenADRClient:
         if event['event_descriptor']['event_id'] in self.events:
             return self.responded_events['event_id']
 
-    async def on_register_report(self, report):
-        """
-        Placeholder for the on_register_report handler.
-        """
-
     ###########################################################################
     #                                                                         #
     #                                  LOW LEVEL                              #
@@ -744,7 +743,7 @@ class OpenADRClient:
                 else:
                     self.responded_events[event_id] = result
             for i, result in enumerate(results):
-                if result not in ('optIn', 'optOut'):
+                if result not in ('optIn', 'optOut') and events[i]['response_required'] == 'always':
                     logger.error("Your on_event or on_update_event handler must return 'optIn' or 'optOut'; "
                                  f"you supplied {result}. Please fix your on_event handler.")
                     results[i] = 'optOut'
@@ -753,29 +752,27 @@ class OpenADRClient:
                          f"The error was {err.__class__.__name__}: {str(err)}")
             results = ['optOut'] * len(events)
 
-        if len(events) == 1:
-            logger.debug(f"Now responding with {results[0]}")
-        else:
-            logger.debug(f"Responding to multiple events: {results}.")
-
         event_responses = [{'response_code': 200,
                             'response_description': 'OK',
                             'opt_type': results[i],
                             'request_id': message['request_id'],
                             'modification_number': 1,
                             'event_id': events[i]['event_descriptor']['event_id']}
-                           for i, event in enumerate(events)]
+                           for i, event in enumerate(events) if event['response_required'] == 'always']
 
-        response = {'response_code': 200,
-                    'response_description': 'OK',
-                    'request_id': message['request_id']}
-        message = self._create_message('oadrCreatedEvent',
-                                       response=response,
-                                       event_responses=event_responses,
-                                       ven_id=self.ven_id)
-        service = 'EiEvent'
-        response_type, response_payload = await self._perform_request(service, message)
-        logger.info(response_type, response_payload)
+        if len(event_responses) > 0:
+            response = {'response_code': 200,
+                        'response_description': 'OK',
+                        'request_id': message['request_id']}
+            message = self._create_message('oadrCreatedEvent',
+                                           response=response,
+                                           event_responses=event_responses,
+                                           ven_id=self.ven_id)
+            service = 'EiEvent'
+            response_type, response_payload = await self._perform_request(service, message)
+            logger.info(response_type, response_payload)
+        else:
+            logger.info("Not sending any event responses, because a response was not required/allowed by the VTN.")
 
     async def _poll(self):
         logger.debug("Now polling for new messages")
@@ -803,11 +800,6 @@ class OpenADRClient:
                 for report_request in response_payload['report_requests']:
                     await self.create_report(report_request)
 
-        elif response_type == 'oadrRegisterReport':
-            if 'reports' in response_payload and len(response_payload['reports']) > 0:
-                for report in response_payload['reports']:
-                    await self.register_report(report)
-
         else:
             logger.warning(f"No handler implemented for incoming message "
                            f"of type {response_type}, ignoring.")
