@@ -23,7 +23,6 @@ from lxml.etree import Element
 from asyncio import iscoroutine
 from openleadr import errors
 from datetime import datetime, timezone, timedelta
-from openleadr.utils import ensure_bytes
 import os
 
 from openleadr import utils
@@ -80,7 +79,6 @@ def create_message(message_type, cert=None, key=None, passphrase=None, **message
     msg = envelope.render(template=f'{message_type}',
                           signature=signature,
                           signed_object=signed_object)
-    validate_xml_schema(ensure_bytes(msg))
     return msg
 
 
