@@ -19,7 +19,6 @@ from dataclasses import is_dataclass, asdict
 from collections import OrderedDict
 from openleadr import enums, objects
 import asyncio
-import itertools
 import re
 import ssl
 import hashlib
@@ -39,24 +38,6 @@ def generate_id(*args, **kwargs):
     return str(uuid.uuid4())
 
 
-def indent_xml(message):
-    """
-    Indents the XML in a nice way.
-    """
-    INDENT_SIZE = 2
-    lines = [line.strip() for line in message.split("\n") if line.strip() != ""]
-    indent = 0
-    for i, line in enumerate(lines):
-        if i == 0:
-            continue
-        if re.search(r'^</[^>]+>$', line):
-            indent = indent - INDENT_SIZE
-        lines[i] = " " * indent + line
-        if not (re.search(r'</[^>]+>$', line) or line.endswith("/>")):
-            indent = indent + INDENT_SIZE
-    return "\n".join(lines)
-
-
 def flatten_xml(message):
     """
     Flatten the entire XML structure.
@@ -286,27 +267,25 @@ def parse_duration(value):
     """
     Parse a RFC5545 duration.
     """
-    # TODO: implement the full regex:
-    # matches = re.match(r'(\+|\-)?P((\d+Y)?(\d+M)?(\d+D)?T?(\d+H)?(\d+M)?(\d+S)?)|(\d+W)', value)
     if isinstance(value, timedelta):
         return value
-    matches = re.match(r'P(\d+(?:D|W))?T?(\d+H)?(\d+M)?(\d+S)?', value)
+    regex = r'(\+|\-)?P(?:(?:(\d+)Y)?(?:(\d+)M)?(?:(\d+)D)?T?(?:(\d+)H)?(?:(\d+)M)?(?:(\d+)S)?)|(?:(\d+)W)'
+    matches = re.match(regex, value)
     if not matches:
-        return False
-    days = hours = minutes = seconds = 0
-    _days, _hours, _minutes, _seconds = matches.groups()
-    if _days:
-        if _days.endswith("D"):
-            days = int(_days[:-1])
-        elif _days.endswith("W"):
-            days = int(_days[:-1]) * 7
-    if _hours:
-        hours = int(_hours[:-1])
-    if _minutes:
-        minutes = int(_minutes[:-1])
-    if _seconds:
-        seconds = int(_seconds[:-1])
-    return timedelta(days=days, hours=hours, minutes=minutes, seconds=seconds)
+        raise ValueError(f"The duration '{value}' did not match the requested format")
+    years, months, days, hours, minutes, seconds, weeks = (int(g) if g else 0 for g in matches.groups()[1:])
+    if years != 0:
+        logger.warning("Received a duration that specifies years, which is not a determinate duration. "
+                       "It will be interpreted as 1 year = 365 days.")
+        days = days + 365 * years
+    if months != 0:
+        logger.warning("Received a duration that specifies months, which is not a determinate duration "
+                       "It will be interpreted as 1 month = 30 days.")
+        days = days + 30 * months
+    duration = timedelta(weeks=weeks, days=days, hours=hours, minutes=minutes, seconds=seconds)
+    if matches.groups()[0] == "-":
+        duration = -1 * duration
+    return duration
 
 
 def parse_boolean(value):
@@ -316,18 +295,6 @@ def parse_boolean(value):
         return False
 
 
-def peek(iterable):
-    """
-    Peek into an iterable.
-    """
-    try:
-        first = next(iterable)
-    except StopIteration:
-        return None
-    else:
-        return itertools.chain([first], iterable)
-
-
 def datetimeformat(value, format=DATETIME_FORMAT):
     """
     Format a given datetime as a UTC ISO3339 string.
@@ -445,10 +412,10 @@ def find_by(dict_or_list, key, value, *args):
             _item = item
         for key, value in search_params:
             if isinstance(value, tuple):
-                if _item[key] not in value:
+                if key not in _item or _item[key] not in value:
                     break
             else:
-                if _item[key] != value:
+                if key not in _item or _item[key] != value:
                     break
         else:
             return item
@@ -507,13 +474,6 @@ def get_cert_fingerprint_from_request(request):
             return certificate_fingerprint_from_der(der_bytes)
 
 
-def get_certificate_common_name(request):
-    cert = request.transport.get_extra_info('peercert')
-    if cert:
-        subject = dict(x[0] for x in cert['subject'])
-        return subject.get('commonName')
-
-
 def group_targets_by_type(list_of_targets):
     targets_by_type = {}
     for target in list_of_targets:
@@ -555,25 +515,25 @@ def validate_report_measurement_dict(measurement):
         required_description = _MEASUREMENT_DESCRIPTIONS[name]
         if description != required_description:
             if description.lower() == required_description.lower():
-                logger.warning(f"The description for the measurement with name {name} "
-                               f"was not in the correct case; you provided {description} but "
-                               f"it should be {required_description}. "
+                logger.warning(f"The description for the measurement with name '{name}' "
+                               f"was not in the correct case; you provided '{description}' but "
+                               f"it should be '{required_description}'. "
                                "This was automatically corrected.")
                 measurement['description'] = required_description
             else:
-                raise ValueError(f"The measurement's description {description} "
+                raise ValueError(f"The measurement's description '{description}' "
                                  f"did not match the expected description for this type "
-                                 f" ({required_description}). Please correct this, or use "
+                                 f" ('{required_description}'). Please correct this, or use "
                                  "'customUnit' as the name.")
         if unit not in _ACCEPTABLE_UNITS[name]:
-            raise ValueError(f"The unit {unit} is not acceptable for measurement {name}. Allowed "
-                             f"units are {_ACCEPTABLE_UNITS[name]}.")
+            raise ValueError(f"The unit '{unit}' is not acceptable for measurement '{name}'. Allowed "
+                             f"units are: '" + "', '".join(_ACCEPTABLE_UNITS[name]) + "'.")
     else:
         if name != 'customUnit':
             logger.warning(f"You provided a measurement with an unknown name {name}. "
                            "This was corrected to 'customUnit'. Please correct this in your "
                            "report definition.")
-            measurement['report_description']['name'] = 'customUnit'
+            measurement['name'] = 'customUnit'
 
     if 'power' in name:
         if 'power_attributes' in measurement:
@@ -622,15 +582,18 @@ def determine_event_status(active_period):
 
 
 async def delayed_call(func, delay):
-    if isinstance(delay, timedelta):
-        delay = delay.total_seconds()
-    await asyncio.sleep(delay)
-    if asyncio.iscoroutinefunction(func):
-        await func()
-    elif asyncio.iscoroutine(func):
-        await func
-    else:
-        func()
+    try:
+        if isinstance(delay, timedelta):
+            delay = delay.total_seconds()
+        await asyncio.sleep(delay)
+        if asyncio.iscoroutinefunction(func):
+            await func()
+        elif asyncio.iscoroutine(func):
+            await func
+        else:
+            func()
+    except asyncio.CancelledError:
+        pass
 
 
 def hasmember(obj, member):
@@ -670,14 +633,11 @@ def get_next_event_from_deque(deque):
     unused_elements = []
     event = None
     for i in range(len(deque)):
-        try:
-            msg = deque.popleft()
-            if isinstance(msg, objects.Event) or (isinstance(msg, dict) and 'event_descriptor' in msg):
-                event = msg
-                break
-            else:
-                unused_elements.append(msg)
-        except IndexError:
-            pass
+        msg = deque.popleft()
+        if isinstance(msg, objects.Event) or (isinstance(msg, dict) and 'event_descriptor' in msg):
+            event = msg
+            break
+        else:
+            unused_elements.append(msg)
     deque.extend(unused_elements)
     return event
